////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Elektrobit Automotive GmbH
// Alle Rechte vorbehalten. All Rights Reserved.
//
// Information contained herein is subject to change without notice.
// Elektrobit retains ownership and all other rights in the software and each
// component thereof.
// Any reproduction of the software or components thereof without the prior
// written permission of Elektrobit is prohibited.
////////////////////////////////////////////////////////////////////////////////

#include <GtfMetaInfo/WidgetDescriptorMacros.h>
#include <GtfType/GtfTypeManager.h>
#include "RippleEffectDesc.h"
#include "RippleEffectWidget.h"

const char* RippleEffectDesc::WidgetName = "RippleEffect";

// pre-defined property types
static const GtfTypeBase s_typeInt32("int32_t");               // int32_t property
static const GtfTypeBase typeFloat("float");                   // float property
static const GtfTypeList typeVec3(typeFloat);                  // vector of 3 int32_t property
static const GtfTypeList typeVec4(typeFloat);                  // vector of 4 int32_t property
static const GtfTypeBase typeString("string");                 // string property
static const GtfTypeTypedef s_typeColor("color", s_typeInt32); // color property. Base type is int32_t
static const GtfTypeResource s_typeImage("image");             // image property as resource type
static const GtfTypeBase s_typeBool("bool");                   // bool property

// Additional properties of the custom widget. Default properties like x, y, 
// width, height, isVisible are already defined in GtfAbstractVisualWidget
static PropertyDescriptor RippleEffectProperties[] =
{
    PropertyDescriptor(&s_typeInt32          // property type
    , "animationTime"                        // property name
    , "The time of the animation"            // property description
    , "Common"                               // property category
    , "0"),                                  // default value
    PropertyDescriptor(&s_typeBool           // property type
    , "enabled"                              // property name
    , "True if the ripple effect is enabled" // property description
    , "Common"                               // property category
    , "true"),                               // default value
};

// The widget descriptor
static const WidgetDescriptor widgets[] =
{ 
    FULL_WIDGET(RippleEffectDesc::WidgetName    // widget name
    , "Ripple effect"                           // widget Alias
    , "A custom widget adding a ripple effect." // widget description
    , "Custom widgets"                          // widget set name (category in EB Guide Studio)
    , "GtfAbstractVisualWidget"                 // base widget
    , false                                     // is abstract (abstract widget can not be used in EB Guide Studio)
    , 0x0100                                    // version, here 1.0
    , false                                     // is view
    , true                                      // can have children
    , false                                     // is instantiator
    , RippleEffectProperties                    // array of widget properties
    , createRippleEffect                        // creation method
    , destroyRippleEffect),                     // destroy method
};

ActionDescriptor const* RippleEffectDesc::GetActions(uint32_t& count_) const
{
    count_ = 0;
    return NULL;
}

PopupStackDescriptor const* RippleEffectDesc::GetPopupStacks(uint32_t& count_) const
{
    count_ = 0;
    return NULL;
}

MediatorDescriptor const* RippleEffectDesc::GetMediators(uint32_t& count_) const
{
    count_ = 0;
    return NULL;
}

WidgetDescriptor const* RippleEffectDesc::GetWidgets(uint32_t& count_) const
{
    count_ = ARRAY_SIZE(widgets);
    return widgets;
}

WidgetFeatureDescriptor const* RippleEffectDesc::GetWidgetFeatures(uint32_t& count_) const
{
    count_ = 0;
    return NULL;
}

ResourceDescriptor const* RippleEffectDesc::GetResourceTypes(uint32_t& count_) const
{
    count_ = 0;
    return NULL;
}

// Function to resolve the types used by the custom properties
bool RippleEffectDesc::ResolveTypes(GtfTypeManager const* typeManager_)
{
    bool success = true;
    for (size_t propertyTypeIndex = 0; propertyTypeIndex < ARRAY_SIZE(RippleEffectProperties); ++propertyTypeIndex)
    {
        PropertyDescriptor *curProperty = &(RippleEffectProperties[propertyTypeIndex]);
        curProperty->m_pType = typeManager_->getTypeByType(curProperty->m_pUnresolvedType);
        success = success && curProperty->m_pType;
    }
    return success;
}
