////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Elektrobit Automotive GmbH
// Alle Rechte vorbehalten. All Rights Reserved.
//
// Information contained herein is subject to change without notice.
// Elektrobit retains ownership and all other rights in the software and each
// component thereof.
// Any reproduction of the software or components thereof without the prior
// written permission of Elektrobit is prohibited.
////////////////////////////////////////////////////////////////////////////////

#ifndef TRIANGLE_WIDGET_H_INCLUDED
#define TRIANGLE_WIDGET_H_INCLUDED

#include <GtfWidgetSet/GtfWidgetModel.h>
#include <GtfTypes/GtfNoncopyable.h>
#include <GtfViewFactory/GtfViewFactory.h>
#include <GtfWidgetSet/GtfAbstractWidget.h>

class GtfWidgetModel;
class GtfWidgetContext;

extern GtfAbstractWidget* createTriangleWidget(GtfWidgetContext const& context_, GtfWidgetModel& model_);
extern void destroyTriangleWidget(GtfAbstractWidget* widget_);

class TriangleWidget : public GtfAbstractWidget, GtfNoncopyable
{
public:

    //!
    //! \brief Constructor
    //!
    //! \param model_[in] The widget model.
    //! \param context_[in] The widget context.
    //!
    TriangleWidget(GtfWidgetModel& model_, GtfWidgetContext const& context_);

    //!
    //! \brief Destructor
    //!
    ~TriangleWidget();

    //!
    //! \brief Method called once in the widget's lifetime.
    //!
    //! \param context_[in] The widget context
    //!
    //! \details Possibly there are more calls if there are down transitions in the statemachine
    //!          In this case this is a (re)init.
    //!
    virtual void init(GtfWidgetContext const& context_);

    //!
    //! \brief Method called once in the widget's lifetime.
    //!
    //! \param context_[in] The widget context
    //!
    //! \details It is called after the widget tree, virtual and link properties are initialized.
    //!
    virtual void prepare(GtfWidgetContext const& context_);

    //!
    //! \brief Method called after leaving the view.
    //!
    //! \param context_[in] The widget context.
    //!
    virtual void cleanup(GtfWidgetContext const& context_);
};

#endif // TRIANGLE_WIDGET_H_INCLUDED